@@ -4,20 +4,20 @@ Set up a catalog for Axiom assets.
 
 
 from operator import itemgetter
-from typing import Optional
+from typing import MutableMapping, Optional, Union
 
 import pandas as pd
 import requests
 
+from cf_pandas import astype
 from intake.catalog.base import Catalog
 from intake.catalog.local import LocalCatalogEntry
 from intake.source.csv import CSVSource
 from intake_parquet.source import ParquetSource
-from intake_xarray.netcdf import NetCDFSource
 from shapely import wkt
 
 from . import __version__
-from .utils import match_key_to_parameter
+from .utils import match_key_to_parameter, match_std_names_to_parameter
 
 
 search_headers = {"Accept": "application/json"}
@@ -36,18 +36,15 @@ class AXDSCatalog(Catalog):
     def __init__(
         self,
         datatype: str = "platform2",
-        outtype: str = "dataframe",
-        dataframe_filetype: str = "csv",
-        keys_to_match: Optional[str] = None,
-        kwargs_search=None,
-        # kwargs_search: Optional[Dict[str, Union[str, int, float]]] = None,
+        keys_to_match: Optional[Union[str, list]] = None,
+        standard_names: Optional[Union[str, list]] = None,
+        kwargs_search: MutableMapping[str, Union[str, int, float]] = None,
         page_size: int = 10,
         verbose: bool = False,
         name: str = "catalog",
         description: str = "Catalog of Axiom assets.",
-        metadata=None,
-        # metadata: Optional[Dict[str, Union[str, int, float]]] = None,
-        ttl: Optional[int] = None,
+        metadata: dict = None,
+        ttl: int = 86400,
         **kwargs,
     ):
         """Initialize an Axiom Catalog.
@@ -55,24 +52,22 @@ class AXDSCatalog(Catalog):
         Parameters
         ----------
         datatype : str
-            Axiom data type. Currently only "platform2" but eventually also "module".
-        outtype : str
-            Type of output. Probably will be "dataframe" or "xarray".
-        dataframe_filetype : str, optional
-            Type of file data for dataframe comes from: "csv" (default) or "parquet". About half of the datasets do not have parquet files currently — you can see them by inputting `verbose=True` to your call.
-        keys_to_match : str, optional
-            Name of key to match with system-available variable parameterNames using criteria. Currently only 1 at a time.
+            Axiom data type. Currently only "platform2" but eventually also "module". Platforms will be returned as dataframe containers.
+        keys_to_match : str, list, optional
+            Name of keys to match with system-available variable parameterNames using criteria. To filter search by variables, either input keys_to_match and a vocabulary or input standard_names.
+        standard_names : str, list, optional
+            Standard names to select from Axiom search parameterNames. To filter search by variables, either input keys_to_match and a vocabulary or input standard_names.
         kwargs_search : dict, optional
             Keyword arguments to input to search on the server before making the catalog. Options are:
             * to search by bounding box: include all of min_lon, max_lon, min_lat, max_lat: (int, float). Longitudes must be between -180 to +180.
             * to search within a datetime range: include both of min_time, max_time: interpretable datetime string, e.g., "2021-1-1"
             * to search using a textual keyword: include `search_for` as a string.
         page_size : int, optional
-            Number of results. Default is 1000 but fewer is faster.
+            Number of results. Fewer is faster. Note that default is 10. Note that if you want to make sure you get all available datasets, you should input a large number like 50000.
         verbose : bool, optional
             Set to True for helpful information.
         ttl : int, optional
-            Time to live for catalog. How long before force-reloading catalog. Set to None to not do this.
+            Time to live for catalog (in seconds). How long before force-reloading catalog. Set to None to not do this. Currently default is set to a large number because the available version of intake does not have a change to accept None.
         name : str, optional
             Name for catalog.
         description : str, optional
@@ -88,14 +83,6 @@ class AXDSCatalog(Catalog):
         self.kwargs_search = kwargs_search
         self.page_size = page_size
         self.verbose = verbose
-        self.dataframe_filetype = dataframe_filetype
-
-        if datatype == "module" and outtype == "dataframe":
-            raise ValueError(
-                "modules cannot be exported as dataframes since they are gridded data."
-            )
-
-        self.outtype = outtype
 
         if kwargs_search is not None:
             checks = [
@@ -109,94 +96,117 @@ class AXDSCatalog(Catalog):
                     raise ValueError(
                         f"If any of {check} are input, they all must be input."
                     )
-            if (
-                abs(kwargs_search["min_lon"]) > 180
-                or abs(kwargs_search["max_lon"]) > 180
-            ):
-                raise ValueError(
-                    "`min_lon` and `max_lon` must be in the range -180 to 180."
-                )
+
+            if "min_lon" in kwargs_search and "max_lon" in kwargs_search:
+                min_lon, max_lon = kwargs_search["min_lon"], kwargs_search["max_lon"]
+                if isinstance(min_lon, (int, float)) and isinstance(
+                    max_lon, (int, float)
+                ):
+                    if abs(min_lon) > 180 or abs(max_lon) > 180:
+                        raise ValueError(
+                            "`min_lon` and `max_lon` must be in the range -180 to 180."
+                        )
+
         else:
             kwargs_search = {}
         self.kwargs_search = kwargs_search
 
+        # input keys_to_match OR standard_names but not both
+        if keys_to_match is not None and standard_names is not None:
+            raise ValueError(
+                "Input either `keys_to_match` or `standard_names` but not both."
+            )
+
+        self.pglabels: Optional[list] = None
         if keys_to_match is not None:
-            # Currently just take first match, but there could be more than one.
-            self.pglabel = match_key_to_parameter(keys_to_match)[0]
-        else:
-            self.pglabel = None
+            self.pglabels = match_key_to_parameter(astype(keys_to_match, list))
+        elif standard_names is not None:
+            self.pglabels = match_std_names_to_parameter(astype(standard_names, list))
 
         # Put together catalog-level stuff
         if metadata is None:
             metadata = {}
             metadata["kwargs_search"] = self.kwargs_search
-            metadata["pglabel"] = self.pglabel
-            metadata["outtype"] = self.outtype
+            metadata["pglabels"] = self.pglabels
 
         super(AXDSCatalog, self).__init__(
             **kwargs, ttl=ttl, name=name, description=description, metadata=metadata
         )
 
-    @property
-    def search_url(self):
+    def search_url(self, pglabel: Optional[str] = None) -> str:
         """Set up url for search."""
 
-        if not hasattr(self, "_search_url"):
+        self.url_search_base = f"https://search.axds.co/v2/search?portalId=-1&page=1&pageSize={self.page_size}&verbose=true"
 
-            self.url_search_base = f"https://search.axds.co/v2/search?portalId=-1&page=1&pageSize={self.page_size}&verbose=true"
+        url = f"{self.url_search_base}&type={self.datatype}"
 
-            url = f"{self.url_search_base}&type={self.datatype}"
+        assert isinstance(self.kwargs_search, dict)
+        if self.kwargs_search.keys() >= {
+            "max_lon",
+            "min_lon",
+            "min_lat",
+            "max_lat",
+        }:
+            url_add_box = (
+                f'&geom={{"type":"Polygon","coordinates":[[[{self.kwargs_search["min_lon"]},{self.kwargs_search["min_lat"]}],'
+                + f'[{self.kwargs_search["max_lon"]},{self.kwargs_search["min_lat"]}],'
+                + f'[{self.kwargs_search["max_lon"]},{self.kwargs_search["max_lat"]}],'
+                + f'[{self.kwargs_search["min_lon"]},{self.kwargs_search["max_lat"]}],'
+                + f'[{self.kwargs_search["min_lon"]},{self.kwargs_search["min_lat"]}]]]}}'
+            )
+            url += f"{url_add_box}"
 
-            if self.kwargs_search.keys() >= {
-                "max_lon",
-                "min_lon",
-                "min_lat",
-                "max_lat",
-            }:
-                url_add_box = (
-                    f'&geom={{"type":"Polygon","coordinates":[[[{self.kwargs_search["min_lon"]},{self.kwargs_search["min_lat"]}],'
-                    + f'[{self.kwargs_search["max_lon"]},{self.kwargs_search["min_lat"]}],'
-                    + f'[{self.kwargs_search["max_lon"]},{self.kwargs_search["max_lat"]}],'
-                    + f'[{self.kwargs_search["min_lon"]},{self.kwargs_search["max_lat"]}],'
-                    + f'[{self.kwargs_search["min_lon"]},{self.kwargs_search["min_lat"]}]]]}}'
-                )
-                url += f"{url_add_box}"
+        if self.kwargs_search.keys() >= {"max_time", "min_time"}:
+            # convert input datetime to seconds since 1970
+            startDateTime = (
+                pd.Timestamp(self.kwargs_search["min_time"]).tz_localize("UTC")
+                - pd.Timestamp("1970-01-01 00:00").tz_localize("UTC")
+            ) // pd.Timedelta("1s")
+            endDateTime = (
+                pd.Timestamp(self.kwargs_search["max_time"]).tz_localize("UTC")
+                - pd.Timestamp("1970-01-01 00:00").tz_localize("UTC")
+            ) // pd.Timedelta("1s")
 
-            if self.kwargs_search.keys() >= {"max_time", "min_time"}:
-                # convert input datetime to seconds since 1970
-                startDateTime = (
-                    pd.Timestamp(self.kwargs_search["min_time"]).tz_localize("UTC")
-                    - pd.Timestamp("1970-01-01 00:00").tz_localize("UTC")
-                ) // pd.Timedelta("1s")
-                endDateTime = (
-                    pd.Timestamp(self.kwargs_search["max_time"]).tz_localize("UTC")
-                    - pd.Timestamp("1970-01-01 00:00").tz_localize("UTC")
-                ) // pd.Timedelta("1s")
+            # search by time
+            url_add_time = f"&startDateTime={startDateTime}&endDateTime={endDateTime}"
 
-                # search by time
-                url_add_time = (
-                    f"&startDateTime={startDateTime}&endDateTime={endDateTime}"
-                )
+            url += f"{url_add_time}"
 
-                url += f"{url_add_time}"
+        if "search_for" in self.kwargs_search:
+            url += f"&query={self.kwargs_search['search_for']}"
 
-            if "search_for" in self.kwargs_search:
-                url += f"&query={self.kwargs_search['search_for']}"
+        # if self.pglabel is not None:
 
-            if self.pglabel is not None:
+        # search by variable
+        if pglabel is not None:
+            url += f"&tag=Parameter+Group:{pglabel}"
 
-                # search by variable
-                url += f"&tag=Parameter+Group:{self.pglabel}"
+        # if requests.get(url).status_code != 200:
+        #     raise ValueError("")
 
-            # if requests.get(url).status_code != 200:
-            #     raise ValueError("")
+        if self.verbose:
+            print(f"search url: {url}")
 
-            if self.verbose:
-                print(f"search url: {url}")
+        return url
 
-            self._search_url = url
+    def get_search_urls(self) -> list:
+        """Gather all search urls for catalog.
 
-        return self._search_url
+        Returns
+        -------
+        list
+            List of search urls.
+        """
+
+        if self.pglabels is not None:
+            search_urls = []
+            for pglabel in self.pglabels:
+                # import pdb; pdb.set_trace()
+                search_urls.append(self.search_url(pglabel))
+        else:
+            search_urls = [self.search_url()]
+
+        return search_urls
 
     def _load_metadata(self, results) -> dict:  #: Dict[str, str]
         """Load metadata for catalog entry.
@@ -247,26 +257,45 @@ class AXDSCatalog(Catalog):
 
         return metadata
 
+    def _load_all_results(self):
+
+        all_results = []
+        for search_url in self.get_search_urls():
+            res = requests.get(search_url, headers=search_headers).json()
+            if "results" not in res:
+                raise ValueError(
+                    f"No results were returned for the search. Search url: {search_url}."
+                )
+
+            if self.verbose:
+                print(
+                    f"For search url {search_url}, number of results found: {len(res['results'])}. Page size: {self.page_size}."
+                )
+
+            all_results.extend(res["results"])
+        return all_results
+
     def _load(self):
         """Find all dataset ids and create catalog."""
 
-        res = requests.get(self.search_url, headers=search_headers).json()
-
-        if "results" not in res:
-            raise ValueError("No results were returned for the search.")
+        results = self._load_all_results()
 
         if self.verbose:
+            unique = set([res["uuid"] for res in results])
             print(
-                f"Number of results found: {len(res['results'])}. Page size: {self.page_size}."
+                f"Total number of results found: {len(results)}, but unique results: {len(unique)}."
             )
 
         self._entries = {}
-        missing_datasets = []  # dataset_ids that don't have requested outtype
-        for results in res["results"]:
+        for results in results:
             dataset_id = results["uuid"]
 
-            # if self.verbose:
-            #     print("Dataset id: ", dataset_id)
+            # don't repeat an entry (it won't actually allow you to, but probably saves time not to try)
+            if dataset_id in self._entries:
+                continue
+
+            if self.verbose:
+                print(f"Dataset ID: {dataset_id}")
 
             # # quick check if OPENDAP is in the access methods for this uuid, otherwise move on
             # if self.datatype == "module":
@@ -290,57 +319,21 @@ class AXDSCatalog(Catalog):
 
             description = f"AXDS dataset_id {dataset_id} of datatype {self.datatype}"
 
-            # docs = return_docs_response(dataset_id)
             # Find urlpath
             if self.datatype == "platform2":
-                # urlpath = results["source"]["files"]["data.csv.gz"]["url"]
-                # urlpath = results["source"]["files"]["deployment.nc"]["url"]
-                # urlpath = results["source"]["files"]["data.viz.parquet"]["url"]
-                # url = f"{self.url_docs_base}&id={dataset_id}"
-                # res2 = requests.get(url, headers=search_headers).json()
-                if self.outtype == "dataframe":
-                    if self.dataframe_filetype == "csv":
-                        # urlpath = docs["data"]["resources"]["files"]["data.csv.gz"]["url"]
-                        urlpath = results["source"]["files"]["data.csv.gz"]["url"]
-                        plugin = CSVSource  # 'csv'
-                    elif self.dataframe_filetype == "parquet":
-                        try:
-                            key = [
-                                key
-                                for key in results["source"]["files"].keys()
-                                if ".parquet" in key
-                            ][0]
-                            urlpath = results["source"]["files"][key]["url"]
-                            # urlpath = results["source"]["files"]["data.viz.parquet"]["url"]
-                        except Exception:
-                            missing_datasets.append(dataset_id)
-                            continue
-                        plugin = ParquetSource
-                    else:
-                        raise ValueError(
-                            f"Valid values of `dataframe_filetype` are 'csv' and 'parquet'. User entered {self.dataframe_filetype}."
-                        )
-                elif self.outtype == "xarray":
-                    # key = [
-                    #     key
-                    #     for key in docs["data"]["resources"]["files"].keys()
-                    #     if ".nc" in key
-                    # ][0]
-                    # urlpath = docs["data"]["resources"]["files"][key]["url"]
-                    try:
-                        key = [
-                            key
-                            for key in results["source"]["files"].keys()
-                            if ".nc" in key
-                        ][0]
-                        urlpath = results["source"]["files"][key]["url"]
-                    except Exception:
-                        # if self.verbose:
-                        #     print(f"Dataset has no netcdf file: {dataset_id}.")
-                        missing_datasets.append(dataset_id)
-                        continue
-                    # urlpath = results["source"]["files"]["deployment.nc"]["url"]
-                    plugin = NetCDFSource  # 'netcdf'
+                # use parquet if available, otherwise csv
+                try:
+                    key = [
+                        key
+                        for key in results["source"]["files"].keys()
+                        if ".parquet" in key
+                    ][0]
+                    urlpath = results["source"]["files"][key]["url"]
+                    plugin = ParquetSource
+                except Exception:
+                    urlpath = results["source"]["files"]["data.csv.gz"]["url"]
+                    plugin = CSVSource
+
             # elif self.datatype == "module":
             #     plugin = NetCDFSource  # 'netcdf'
 
@@ -416,8 +409,3 @@ class AXDSCatalog(Catalog):
             entry._plugin = [plugin]
 
             self._entries[dataset_id] = entry
-        if self.verbose:
-            # if self.outtype == "xarray":
-            print(
-                f"Number of datasets with file missing for outtype {self.outtype}: {len(missing_datasets)}. The dataset_ids have been excluded from the catalog and are {missing_datasets}."
-            )
